const canvas = document.getElementById('gameCanvas');
const ctx = canvas.getContext('2d');
const box = 20;
let snake = [{ x: 9 * box, y: 9 * box }];
let direction = 'RIGHT';
let food = {};
let pendingDirection = '';
let gameOver = false;
let score = 0;
let gameInterval;
let scoreElement = document.querySelector('.score');

document.addEventListener('keydown', directionControl);

function directionControl(event) {
  const keyPressed = event.key;
  if (keyPressed === 'ArrowLeft' && direction !== 'RIGHT') {
    pendingDirection = 'LEFT';
  } else if (keyPressed === 'ArrowUp' && direction !== 'DOWN') {
    pendingDirection = 'UP';
  } else if (keyPressed === 'ArrowRight' && direction !== 'LEFT') {
    pendingDirection = 'RIGHT';
  } else if (keyPressed === 'ArrowDown' && direction !== 'UP') {
    pendingDirection = 'DOWN';
  } else if (keyPressed === 'a' && direction !== 'RIGHT') {
    pendingDirection = 'LEFT';
  } else if (keyPressed === 'w' && direction !== 'DOWN') {
    pendingDirection = 'UP';
  } else if (keyPressed === 'd' && direction !== 'LEFT') {
    pendingDirection = 'RIGHT';
  } else if (keyPressed === 's' && direction !== 'UP') {
    pendingDirection = 'DOWN';
  } else if (keyPressed === 'r' && gameOver) {
    resetGame();
  }
}

function draw() {
  ctx.fillStyle = 'black';
  ctx.fillRect(0, 0, 400, 400);

  // Draw Checkerboard Background
  for (let i = 0; i < 20; i++) {
    for (let j = 0; j < 20; j++) {
      if ((i + j) % 2 === 0) {
        ctx.fillStyle = '#4CAF50'; // Dark Green
        ctx.fillRect(i * box, j * box, box, box);
      } else {
        ctx.fillStyle = '#8BC34A'; // Light Green
        ctx.fillRect(i * box, j * box, box, box);
      }
    }
  }

  // Draw Snake
  ctx.fillStyle = 'green';
  ctx.fillRect(snake[0].x, snake[0].y, box, box);

  // Draw Snake's Head Direction
  ctx.fillStyle = 'green';
  ctx.beginPath();
  if (direction === 'RIGHT') {
    ctx.arc(
      snake[0].x + box / 2,
      snake[0].y + box / 2,
      box / 2,
      (Math.PI * 3) / 2,
      (Math.PI * 5) / 2
    );
  } else if (direction === 'LEFT') {
    ctx.arc(
      snake[0].x + box / 2,
      snake[0].y + box / 2,
      box / 2,
      Math.PI / 2,
      (Math.PI * 3) / 2
    );
  } else if (direction === 'UP') {
    ctx.arc(
      snake[0].x + box / 2,
      snake[0].y + box / 2,
      box / 2,
      Math.PI,
      Math.PI * 2
    );
  } else if (direction === 'DOWN') {
    ctx.arc(snake[0].x + box / 2, snake[0].y + box / 2, box / 2, 0, Math.PI);
  }
  ctx.fill();

  // Draw Snake's Body
  for (let i = 1; i < snake.length; i++) {
    ctx.fillRect(snake[i].x, snake[i].y, box, box);
  }

  // Draw Food
  ctx.fillStyle = 'red';
  ctx.fillRect(food.x, food.y, box, box);

  // Update Snake Direction
  if (pendingDirection) {
    direction = pendingDirection;
    pendingDirection = '';
  }

  // Snake Position
  let snakeX = snake[0].x;
  let snakeY = snake[0].y;

  // Change Snake Position
  if (direction == 'LEFT') snakeX -= box;
  if (direction == 'UP') snakeY -= box;
  if (direction == 'RIGHT') snakeX += box;
  if (direction == 'DOWN') snakeY += box;

  // Check for Wall Collisions
  if (
    snakeX < 0 ||
    snakeX >= canvas.width ||
    snakeY < 0 ||
    snakeY >= canvas.height
  ) {
    gameOver = true;
    handleGameOver();
  }

  // If Snake Eats Food
  if (snakeX === food.x && snakeY === food.y) {
    score++;
    scoreElement.textContent = 'Score: ' + score;
    food = generateFood();
  } else {
    snake.pop();
  }

  // Add new Head
  const newHead = { x: snakeX, y: snakeY };

  // Game Over Conditions
  if (collision(newHead, snake)) {
    gameOver = true;
    handleGameOver();
  }

  snake.unshift(newHead);

  // Game Over Display
  if (gameOver) {
    ctx.font = '30px Arial';
    ctx.fillStyle = 'red';
    ctx.fillText('Game Over!', canvas.width / 2 - 80, canvas.height / 2);
  }
}

function collision(head, array) {
  for (let i = 1; i < array.length; i++) {
    if (head.x === array[i].x && head.y === array[i].y) {
      return true;
    }
  }
  return false;
}

function generateFood() {
  let x, y;
  do {
    x = Math.floor(Math.random() * 20) * box;
    y = Math.floor(Math.random() * 20) * box;
  } while (collision({ x: x, y: y }, snake));
  return { x: x, y: y };
}

function startGame() {
  clearInterval(gameInterval);
  food = generateFood();
  gameInterval = setInterval(draw, 100);
}

// Style
const body = document.querySelector('body');
body.style.backgroundColor = '#111';
body.style.fontFamily = 'Arial, sans-serif';
body.style.display = 'flex';
body.style.justifyContent = 'center';
body.style.alignItems = 'center';
body.style.minHeight = '100vh';
body.style.margin = 0;
body.style.overflow = 'hidden';

canvas.style.border = '2px solid #fff';
canvas.style.boxShadow = '0 0 10px rgba(255, 255, 255, 0.5)';

const backButton = document.querySelector('.back-button');
backButton.style.position = 'absolute';
backButton.style.left = '10px';
backButton.style.top = '10px';
backButton.style.padding = '10px 20px';
backButton.style.backgroundColor = '#4caf50';
backButton.style.color = 'white';
backButton.style.textDecoration = 'none';
backButton.style.borderRadius = '5px';
backButton.style.boxShadow = '0 0 5px rgba(0, 0, 0, 0.5)';

const gameContainer = document.querySelector('canvas');
let gameOverOverlay;
function showGameOver() {
  gameOverOverlay = document.createElement('div');
  gameOverOverlay.style.position = 'absolute';
  gameOverOverlay.style.top = '0';
  gameOverOverlay.style.left = '0';
  gameOverOverlay.style.width = '100%';
  gameOverOverlay.style.height = '100%';
  gameOverOverlay.style.backgroundColor = 'rgba(0, 0, 0, 0.7)';
  gameOverOverlay.style.display = 'flex';
  gameOverOverlay.style.justifyContent = 'center';
  gameOverOverlay.style.alignItems = 'center';
  gameOverOverlay.style.color = 'white';
  gameOverOverlay.style.fontSize = '30px';
  gameOverOverlay.textContent = 'Game Over! Press "R" to restart.';
  gameContainer.appendChild(gameOverOverlay);
  clearInterval(gameInterval);
}

function handleGameOver() {
  showGameOver();
  document.addEventListener('keydown', (event) => {
    if (event.key === 'r') {
      resetGame();
    }
  });
}

function resetGame() {
  if (gameOverOverlay) {
    gameOverOverlay.remove();
  }
  gameOver = false;
  snake = [{ x: 9 * box, y: 9 * box }];
  direction = 'RIGHT';
  pendingDirection = '';
  food = generateFood();
  score = 0;
  scoreElement.textContent = 'Score: ' + score;
  startGame();
}

setTimeout(() => {
  startGame();
}, 3000);
